"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.correlationKey = exports.normalize = exports.forCollectedEvents = exports.correlate = void 0;

var _get = _interopRequireDefault(require("lodash/get"));

var _isFunction = _interopRequireDefault(require("lodash/isFunction"));

var _utils = require("../utils");

var _filters = require("../filters");

var _dynamodb = require("../utils/dynamodb");

/**
 * used when you need to correlate the same events under multiple keys
 * used in trigger functions
 * inserts event back into the micro event store under different correlation keys
 * uses the DynamoDB single table pattern
 *
 * interface Rule {
 *   id: string
 *   flavor: correlate,
 *   eventType: string | string[] | Function,
 *   filters: Function[],
 *   correlationKey: string | Function,
 *   correlationKeySuffix?: string, // used to create alternate correlations on the same key
 *   ttl?: number, // default ttl of collected event
 *   expire: boolean | string
 *   parallel?: number;
 * }
 */
const correlate = rule => s => s // eslint-disable-line import/prefer-default-export
.filter(forCollectedEvents).map(normalize).filter(onEventType(rule)).tap(_utils.printStartPipeline).filter(onContent(rule)).map(correlationKey(rule)).map(toPutRequest(rule)).through((0, _dynamodb.put)(rule)).tap(_utils.printEndPipeline);

exports.correlate = correlate;

const forCollectedEvents = uow => uow.record.eventName === 'INSERT' && uow.record.dynamodb.Keys.sk.S === 'EVENT';

exports.forCollectedEvents = forCollectedEvents;

const normalize = uow => ({ ...uow,
  meta: {
    sequenceNumber: uow.event.raw.new.sequenceNumber,
    ttl: uow.event.raw.new.ttl,
    data: uow.event.raw.new.data
  },
  event: uow.event.raw.new.event
});

exports.normalize = normalize;

const onEventType = rule => (0, _utils.faulty)(uow => (0, _filters.filterOnEventType)(rule, uow));

const onContent = rule => (0, _utils.faulty)(uow => (0, _filters.filterOnContent)(rule, uow));

const correlationKey = rule => (0, _utils.faulty)(uow => {
  let key;

  if ((0, _isFunction.default)(rule.correlationKey)) {
    key = rule.correlationKey(uow);
  } else {
    key = (0, _get.default)(uow.event, rule.correlationKey);
  } // use a suffix when you need the same key for different sets of rules


  key = rule.correlationKeySuffix ? `${key}.${rule.correlationKeySuffix}` : key;
  return { ...uow,
    key
  };
});

exports.correlationKey = correlationKey;

const toPutRequest = rule => (0, _utils.faulty)(uow => ({ ...uow,
  putRequest: {
    // variable expected by `put` util
    Item: {
      pk: uow.key,
      sk: uow.event.id,
      discriminator: 'CORREL',
      // ATION
      timestamp: uow.event.timestamp,
      sequenceNumber: uow.meta.sequenceNumber,
      ttl: rule.ttl ? (0, _utils.ttlRule)(rule, uow) : uow.meta.ttl,
      expire: rule.expire,
      suffix: rule.correlationKeySuffix,
      ruleId: rule.id,
      event: uow.event
    }
  }
}));